# -*- coding: utf-8 -*-
"""
dl.py
"""

import numpy as np
import torch
from torch.utils.data import DataLoader
#import matplotlib.pyplot as plt

# dataset
# S : S行列, 4d配列
# labels : 誘電率分布, 3d配列
def dataset(ndata, component, E, S, train_ratio):
    # データ数
    assert E.shape[0] == S.shape[0]
    ndata = S.shape[0] if (ndata <= 0 or ndata > S.shape[0]) else ndata
    #print(ndata, S.shape, labels.shape)

    # 4D->3D
    s_r = S[:, 0, :, :]
    s_i = S[:, 1, :, :]
    s_a = np.sqrt(s_r**2 + s_i**2)

    # 正規化
    #print(s_a.mean(), s_a.std())
    s_r = (s_r - s_r.mean()) / s_r.std()
    s_i = (s_i - s_i.mean()) / s_i.std()
    s_a = (s_a - s_a.mean()) / s_a.std()

    # 乱数発生
    rand = torch.rand(ndata)

    # dataset
    resize = (component, S.shape[2], S.shape[3])
    #print(resize)
    train_set, test_set = [], []
    for i in range(ndata):
        #　誘電率分布, tensor化 + 2d->1d
        label = torch.tensor(E[i]).view(-1)

        # S行列
        if component == 1:
            image = s_a[i]
        elif component == 2:
            image = np.concatenate((s_r[i], s_i[i]), axis=0)
        else:
            image = np.concatenate((s_r[i], s_i[i], s_a[i]), axis=0)
            #image = np.concatenate((s_a[i], s_a[i], s_a[i]), axis=0)
        image = image.reshape(resize)
        #print(image.shape)

        # 乱数で分配
        if rand[i] < train_ratio:
            # 訓練データ
            #r = 0.01  # ノイズ
            #image += r * np.random.standard_normal(image.shape)
            image = torch.tensor(image)
            train_set.append([image, label])
        else:
            # テストデータ
            r = 0  # ノイズ
            image += r * np.random.standard_normal(image.shape)
            image = torch.tensor(image)
            test_set.append([image, label])

    return train_set, test_set

# dataloader
def dataloader(train_set, test_set, batch_size=50):
    train_loader, test_loader = None, None
    if train_set is not None and len(train_set) > 0:
        train_loader = DataLoader(train_set, batch_size=batch_size, shuffle=True)
    if test_set is not None and len(test_set) > 0:
        test_loader  = DataLoader(test_set,  batch_size=batch_size, shuffle=False)

    return train_loader, test_loader
