# -*- coding: utf-8 -*-
"""
plotutils.py
"""

import numpy as np
import matplotlib.pyplot as plt

# 等高線図
# mx, my, mstep : 横の数,　縦の数, 関数配列のステップ数
# f[ndata, nx, ny] : 関数配列
# sta[ndata] : タイトル配列
# figtitle : ページタイトル
def contour(figsize, mx, my, mstep, f, vmin, vmax, sta, figtitle):
    plt.figure(figtitle, figsize=figsize, layout='tight')

    num = 0
    for ix in range(mx):
        for iy in range(my):
            fid = (num + 1) * mstep - 1
            if (fid < f.shape[0]):
                pos = ix + (my - 1 - iy) * mx + 1
                #fid = num * mstep
                #print(Ndata, ix, iy, num, pos)
                ax = plt.subplot(my, mx, pos)
                #ax.contourf(np.transpose(f[fid], (1, 0)), vmin=vmin, vmax=vmax, cmap='rainbow')
                ax.pcolormesh(np.transpose(f[fid], (1, 0)), vmin=vmin, vmax=vmax, cmap='rainbow')
                ax.set_aspect('equal')
                ax.set_axis_off()
                if sta is not None:
                    #title = '%d %.4f' % (fid + 1, sta[fid])
                    ax.set_title(sta[fid], fontsize=8, pad=0)
                num += 1

    suptitle = 'data=%d fig=%d step=%d Nx=%d Ny=%d min=%.5f max=%.5f' % \
        (f.shape[0], num, mstep, f.shape[1], f.shape[2], vmin, vmax)
    #print(suptitle)
    plt.suptitle(suptitle)

    plt.show()

def contour2(figsize, mx, my, mstep, f1, f2, vmin, vmax, sta1, sta2, figtitle):
    assert f1.shape == f2.shape
    plt.figure(figtitle, figsize=figsize, layout='tight')

    num = 0
    for ix in range(mx):
        for iy in range(my):
            fid = (num + 1) * mstep - 1
            if (fid < f1.shape[0]):
                pos1 = (2 * ix + 0) + (my - 1 - iy) * 2 * mx + 1
                ax1 = plt.subplot(my, 2 * mx, pos1)
                #ax1.contourf(np.transpose(f1[fid], (1, 0)), vmin=vmin, vmax=vmax, cmap='rainbow', levels=15)
                ax1.pcolormesh(np.transpose(f1[fid], (1, 0)), vmin=vmin, vmax=vmax, cmap='rainbow')
                ax1.set_aspect('equal')
                ax1.set_axis_off()
                if sta1 is not None:
                    ax1.set_title(sta1[fid], fontsize=8, pad=0, color='r')

                pos2 = (2 * ix + 1) + (my - 1 - iy) * 2 * mx + 1
                ax2 = plt.subplot(my, 2 * mx, pos2)
                #ax2.contourf(np.transpose(f2[fid], (1, 0)), vmin=vmin, vmax=vmax, cmap='rainbow', levels=15)
                ax2.pcolormesh(np.transpose(f2[fid], (1, 0)), vmin=vmin, vmax=vmax, cmap='rainbow')
                ax2.set_aspect('equal')
                ax2.set_axis_off()
                if sta2 is not None:
                    ax2.set_title(sta2[fid], fontsize=8, pad=0)

                num += 1

    suptitle = 'total=%d step=%d fig=%dx2 Nx=%d Ny=%d min=%.5f max=%.5f' % \
        (f1.shape[0], mstep, num, f1.shape[1], f1.shape[2], vmin, vmax)
    plt.suptitle(suptitle)

    plt.show()

"""
# 線上分布図
def _plot2d_profile(figtitle, labels, pred):
    fig = plt.figure('profile', figsize=(8, 6), layout='tight')

    ax = fig.add_subplot()
    idata = pred.shape[0] - 1
    for i in range(pred.shape[2]):
        ax.plot(pred[idata, i])
        ax.plot(labels[idata, i])
    #ax.set_xlim(0, pred.shape[2])
    ax.set_ylim(0.5, 2.5)

    plt.show()
"""