# -*- coding: utf-8 -*-
"""
utils.py
"""

import os, sys
import numpy as np
import torch

def readdata(fn):

    if not os.path.isfile(fn):
        print('file %s not found' % fn)
        sys.exit()

    # データ数
    offset = 0
    count = 6
    idata = np.fromfile(fn, dtype='i4', count=count, offset=offset)
    Ndata, Nx, Ny, Nz, NTx, NRx = idata
    #print(Ndata, Nx, Ny, Nz, NTx, NRx)

    # 誘電率の最小・最大
    offset += count * 4
    count = 2
    fdata = np.fromfile(fn, dtype='f4', count=count, offset=offset)
    dmin, dmax = fdata
    #dmin, dmax, smin, smax = fdata
    #print(dmin, dmax, smin, smax)

    # 誘電率、正規化を復元する(u1->f4)
    offset += count * 4
    count = Ndata * Nx * Ny * Nz
    d = np.fromfile(fn, dtype='u1', count=count, offset=offset)
    d = d.astype(np.float32) / 255
    d = dmin + (dmax - dmin) * d
    E = d.reshape(-1, Nx, Ny, Nz).squeeze()
    #print(E.shape)
    #print(np.min(E), np.max(E))

    # S行列、正規化を復元する(u1->f4)
    offset += count * 1
    count = Ndata * 2 * NTx * NRx
    s = np.fromfile(fn, dtype='f4', count=count, offset=offset)
    #s = s.astype(np.float32) / 255
    #s = smin + (smax - smin) * s
    S = s.reshape(-1, 2, NTx, NRx)
    #print(S.shape)
    #print(np.min(S), np.max(S))

    return Ndata, Nx, Ny, Nz, NTx, NRx, E, S

# 乱数初期化
def torch_seed(seed = 123):
    torch.manual_seed(seed)
    torch.cuda.manual_seed(seed)
    torch.backends.cudnn.deterministic = True
    torch.use_deterministic_algorithms = True

# モデルファイルを読み込む
def load_model(model, modelfile, device):
    if not os.path.isfile(modelfile):
        print('file %s not found' % modelfile)
        sys.exit()
    model.load_state_dict(torch.load(modelfile, map_location=device, weights_only=True))
    print("Load %s" % modelfile)

# テストデータのラベルと予測値をファイルに保存する
def save_data(fn, Nx, Ny, Nz, test_set, preds):
    labels = []
    for _, label in test_set:
        labels.append(label.numpy())
    labels = np.array(labels).reshape(-1)

    np.savez(fn, Nx, Ny, Nz, labels, preds)
    print('Save %s' % fn)

# テストデータのラベルと予測値をファイルから読み込む
def load_data(fn):
    d = np.load(fn)
    Nx     = d[d.files[0]]
    Ny     = d[d.files[1]]
    Nz     = d[d.files[2]]
    labels = d[d.files[3]]
    preds  = d[d.files[4]]
    print('Load %s' % fn)

    return Nx, Ny, Nz, labels, preds
