/*
geometry.c
誘電率、導電率分布
任意個数長方形
ランダムに変化: 縦の長さ, 横の長さ, 縦の位置, 横の位置, 誘電率, 導電率
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <assert.h>

extern double urand(int, int64_t *);
extern void getspan(int, int, int, int, int64_t *, int *, int *);

void geometry(int ndata, int nrect, int64_t seed,
	const int lrect[], const float eps[], const float sig[], int nsmot,
	int nx, int ny, int nout,
	float ***epsr, float ***sigm, float eps0, float sig0, int **rect)
{
	assert((nx > 0) && (ny > 0) && (nout > 0) && (nrect >= 0));

	// データに関するループ
	for (int idata = 0; idata < ndata; idata++) {
		float **e = epsr[idata];
		float **s = sigm[idata];

		// 全体を背景媒質で初期化
		for (int i = 0; i < nx; i++) {
		for (int j = 0; j < ny; j++) {
			e[i][j] = eps0;
			s[i][j] = sig0;
		}
		}

		// 誘電率代入
		for (int irect = 0; irect < nrect; irect++) {
			const float r1 = (float)urand(3, &seed);
			const float r2 = (float)urand(3, &seed);
			const float ep = eps[0] + (eps[1] - eps[0]) * r1;
			const float si = sig[0] + (sig[1] - sig[0]) * r2;
			int i0, i1, j0, j1;
			getspan(nx, nout + lrect[2], lrect[0], lrect[1], &seed, &i0, &i1);
			getspan(ny, nout + lrect[2], lrect[0], lrect[1], &seed, &j0, &j1);
			//printf("%d %d %d %d\n", idata, irect, i0, i1);
			//printf("%d %d %d %d\n", idata, irect, j0, j1);
			for (int i = i0; i < i1; i++) {
			for (int j = j0; j < j1; j++) {
				e[i][j] = ep;
				s[i][j] = si;
			}
			}
			// debug, 長方形（図形出力用）
			rect[idata][(4 * irect) + 0] = i0;
			rect[idata][(4 * irect) + 1] = j0;
			rect[idata][(4 * irect) + 2] = i1;
			rect[idata][(4 * irect) + 3] = j1;
		}

		// 内部領域平滑化(5点平均, 複数回可)
		for (int m = 0; m < nsmot; m++) {
			for (int i = nout; i < nx - nout; i++) {
			for (int j = nout; j < ny - nout; j++) {
				e[i][j] = (
				e[i][j] + e[i - 1][j    ] + e[i + 1][j    ]
				        + e[i    ][j - 1] + e[i    ][j + 1]) / 5;
				s[i][j] = (
				s[i][j] + s[i - 1][j    ] + s[i + 1][j    ]
				        + s[i    ][j - 1] + s[i    ][j + 1]) / 5;
			}
			}
		}
	}
}
