/*
misc.c
*/

#include <stdio.h>
#include <math.h>

extern float C;

// メッシュ
void make_mesh(int Nx, int Ny, float Dx, float Dy, float Eps0, float *Dt,
	float *Xn, float *Yn,
	float *dXn, float *dYn, float *dXc, float *dYc)
{
	// 格子点
	for (int i = 0; i <= Nx; i++) {
		Xn[i] = i * Dx;
	}
	for (int j = 0; j <= Ny; j++) {
		Yn[j] = j * Dy;
	}
	//for (int i = 0; i <= Nx; i++) printf("%d %f\n", i, Xn[i]);
	//for (int j = 0; j <= Ny; j++) printf("%d %f\n", j, Yn[j]);

	// タイムステップ
	//const float eps0 = (Eps0 < 1) ? Eps0 : 1;
	//const float v = (float)(C / sqrt(eps0));
	const float v = (float)(C / sqrt(Eps0));
	*Dt = (float)(1 / sqrt(1 / (Dx * Dx) + 1 / (Dy * Dy)) / v);
	//printf("%e %e %e\n", Dx, Dy, *Dt);

	// C * Dt
	const float cdt = C * (*Dt);

	// 格子点に関する因子
	for (int i = 1; i < Nx; i++) {
		dXn[i] = cdt / ((Xn[i + 1] - Xn[i - 1]) / 2);
	}
	for (int j = 1; j < Ny; j++) {
		dYn[j] = cdt / ((Yn[j + 1] - Yn[j - 1]) / 2);
	}
	dXn[0]  = cdt / (Xn[1]  - Xn[0]);
	dYn[0]  = cdt / (Yn[1]  - Yn[0]);
	dXn[Nx] = cdt / (Xn[Nx] - Xn[Nx - 1]);
	dYn[Ny] = cdt / (Yn[Ny] - Yn[Ny - 1]);

	// セル中心に関する因子
	for (int i = 0; i < Nx; i++) {
		dXc[i] = cdt / (Xn[i + 1] - Xn[i]);
	}
	for (int j = 0; j < Ny; j++) {
		dYc[j] = cdt / (Yn[j + 1] - Yn[j]);
	}
	//for (int i = 0; i <= Nx; i++) printf("%d %.5f\n", i, dXn[i]);
	//for (int j = 0; j <= Ny; j++) printf("%d %.5f\n", j, dYn[j]);
	//for (int i = 0; i <  Nx; i++) printf("%d %.5f\n", i, dXc[i]);
	//for (int j = 0; j <  Ny; j++) printf("%d %.5f\n", j, dYc[j]);
}


// メモリーサイズ[MB]
int memory_size(int nthread, int Ndata, int Nx, int Ny, int NTx, int NRx, int ABC, int fig2)
{
	size_t mem = 0;

	mem += Ndata * Nx * Ny * 2 * sizeof(float);  // Epsr, Sigm
	mem += Ndata * NTx * NRx * 2 * sizeof(float);  // S_r, S_i
	mem += nthread * Nx * Ny * 2 * sizeof(float);  // C1Ez, C2Ez
	mem += nthread * (Nx + 2 * ABC) * (Ny + 2 * ABC) * 3 * sizeof(float);  // Ez, Hx, Hy
	mem += (ABC > 0) * nthread * (Nx + 2 * ABC) * (Ny + 2 * ABC) * 2 * sizeof(float);  // Ezx, Ezy
	mem += fig2 * NTx * Nx * Ny * sizeof(float);  // Ea

	return (int)(mem * 1e-6) + 1;
}
