/*
plot2d.c
fdtd2d図形出力, 2D, debug用
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <assert.h>
#include "ev.h"

extern void getminmax3d(int, int, int, int, int, int, int, float ***, float *, float *);
extern void getminmax4d(int, int, int, int, int, float ****, float *, float *);
extern void plot2d_contour(float, float, int, int, float **, float, float, int, int);

// 誘電率分布, 送受信点
void plot2d_g(int mx, int my, int ndata, int nstep, int nx, int ny, int nout,
	float ***d, int color, int grid,
	int ntx, int *itx, int *jtx, int nrx, int *irx, int *jrx, const char fn[])
{
	char str[BUFSIZ], fmt[BUFSIZ];
	const float dm = 0.3f;

	// 最小・最大
	float dmin, dmax;
	getminmax3d(0, ndata, 0, nx, 0, ny, 0, d, &dmin, &dmax);

	// 開始
	const float s = 2;
	const float w = (mx * nx) + ((mx + 1) * s);
	const float h = (my * ny) + ((my + 1) * s);
	ev2d_init(w, h);
	ev2d_newPage();

	// 等高線図, 1データ=1図
	int num = 0;
	for (int px = 0; px < mx; px++) {
	for (int py = 0; py < my; py++) {
		if (num * nstep < ndata) {
			// 原点
			const float x0 = (px * nx) + ((px + 1) * s);
			const float y0 = (py * ny) + ((py + 1) * s);

			// 等高線図
			plot2d_contour(x0, y0, nx, ny, d[num * nstep], dmin, dmax, color, grid);

			// 内部領域
			ev2d_setColor(160, 160, 160);
			ev2d_drawRectangle(x0 + nout, y0 + nout, x0 + nx - nout, y0 + ny - nout);

			// データ番号
			ev2d_setColor(0, 0, 0);
			sprintf(str, "%d", num * nstep + 1);
			ev2d_drawString(x0, y0 + ny - h / 60, h / 60, str);

			// 送信点
			ev2d_setColor(255, 0, 0);
			for (int n = 0; n < ntx; n++) {
				ev2d_fillRectangle(x0 + itx[n] - dm, y0 + jtx[n] - dm, x0 + itx[n] + dm, y0 + jtx[n] + dm);
			}

			// 受信点
			ev2d_setColor(0, 255, 0);
			for (int n = 0; n < nrx; n++) {
				ev2d_fillRectangle(x0 + irx[n] - dm, y0 + jrx[n] - dm, x0 + irx[n] + dm, y0 + jrx[n] + dm);
			}

			// 図++
			num++;
		}
	}
	}

	// 最小・最大
	strcpy(fmt, "data=%d fig=%d step=%d Nx=%d Ny=%d Mx=%d My=%d Tx=%d Rx=%d min=%.4f max=%.4f");
	sprintf(str, fmt, ndata, num, nstep, nx, ny, nx - 2 * nout, ny - 2 * nout, ntx, nrx, dmin, dmax);
	ev2d_setColor(0, 0, 0);
	ev2d_drawString(s, s, h / 40, str);

	// 出力
	ev2d_file(1, fn);
	ev2d_output();
}


// 電界分布
void plot2d_e(int mx, int my, int ntx, int nstep, int nx, int ny, int npage,
	float ****d, int db, int grid, int *itx, int *jtx, int nrect, int **rect, const char fn[])
{
	char str[BUFSIZ], fmt[BUFSIZ];
	const float eps = 1e-12f;
	const float dm = 0.3f;
	assert((mx > 0) && (my > 0) && (ntx > 0) && (nstep > 0) && (nx > 0) && (ny > 0) && (npage > 0));
	//printf("%d %d %d %d %d %d %d\n", mx, my, ntx, nstep, nx, ny, npage);

	// 最小・最大
	float dmin, dmax;
	getminmax4d(npage, ntx, nx, ny, db, d, &dmin, &dmax);
	//printf("%f %f\n", dmin, dmax);
	if (!db) {
		dmin = 0;
	}

	// 開始
	const float s = 2;
	const float w = (mx * nx) + ((mx + 1) * s);
	const float h = (my * ny) + ((my + 1) * s);
	ev2d_init(w, h);

	// 1データ=1ページ
	for (int idata = 0; idata < npage; idata++) {
		// 改ページ
		ev2d_newPage();

		// 等高線図(カラー塗りつぶし), 1送信点=1図
		int num = 0;
		for (int px = 0; px < mx; px++) {
		for (int py = 0; py < my; py++) {
			if (num * nstep < ntx) {
				// 送信点番号
				const int txid = num * nstep;

				// XY配列
				float **f = d[idata][txid];

				// dB
				if (db) {
					for (int i = 0; i < nx; i++) {
					for (int j = 0; j < ny; j++) {
						f[i][j] = 20 * (float)log10(fmax(f[i][j], eps));
					}
					}
				}

				// 原点
				const float x0 = (px * nx) + ((px + 1) * s);
				const float y0 = (py * ny) + ((py + 1) * s);

				// 等高線図(カラー)
				plot2d_contour(x0, y0, nx, ny, f, dmin, dmax, 1, grid);

				// 送信点
				ev2d_setColor(0, 0, 0);
				ev2d_fillRectangle(x0 + itx[txid] - dm, y0 + jtx[txid] - dm, x0 + itx[txid] + dm, y0 + jtx[txid] + dm);

				// 長方形
				ev2d_setColor(0, 0, 0);
				for (int irect = 0; irect < nrect; irect++) {
					int *ptr = &rect[idata][4 * irect];
					ev2d_drawRectangle(x0 + ptr[0], y0 + ptr[1], x0 + ptr[2], y0 + ptr[3]);
				}

				// データ番号
				ev2d_setColor(0, 0, 0);
				sprintf(str, "%d", num * nstep + 1);
				ev2d_drawString(x0, y0 + ny - h / 60, h / 60, str);

				// 図++
				num++;
			}
		}
		}

		// 最小・最大
		strcpy(fmt, db ? "Tx=%d fig=%d step=%d Nx=%d Ny=%d min=%.3fdB max=%.3fdB"
		               : "Tx=%d fig=%d step=%d Nx=%d Ny=%d min=%.6f max=%.6f");
		sprintf(str, fmt, ntx, num, nstep, nx, ny, dmin, dmax);
		ev2d_setColor(0, 0, 0);
		ev2d_drawString(s, s, h / 40, str);

	}

	// 出力
	ev2d_file(1, fn);
	ev2d_output();
}
