/*
writedata.c
*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <assert.h>

extern void getminmax3d(int, int, int, int, int, int, int, float ***, float *, float *);

// ファイル出力
void writedata(const char outfile[], int ndata, int nx, int ny, int nout, int ntx, int nrx,
	float ***d, float ***s_r, float ***s_i)
{
	//const float inf = 1e12f;

	// (1) 誘電率分布(内部領域)

	const int i0 = nout;
	const int i1 = nx - nout;
	const int j0 = nout;
	const int j1 = ny - nout;

	// 最小値・最大値
	float dmin, dmax;
	getminmax3d(0, ndata, i0, i1, j0, j1, 0, d, &dmin, &dmax);
	//printf("%f %f\n", dmin, dmax);

	// 1D配列化+1バイト化
	const int nd = ndata * (i1 - i0) * (j1 - j0);
	unsigned char *d1d = (unsigned char *)malloc(nd * sizeof(unsigned char));
	int id = 0;
	for (int idata = 0; idata < ndata; idata++) {
		for (int i = i0; i < i1; i++) {
		for (int j = j0; j < j1; j++) {
			d1d[id++] = (unsigned char)(255 * (d[idata][i][j] - dmin) / (dmax - dmin));
		}
		}
	}
	assert(id == nd);

	// (2) S行列
/*
	// 最小値・最大値
	float s1min, s1max, s2min, s2max;
	getminmax3d(0, ndata, 0, ntx, 0, nrx, 0, s_r, &s1min, &s1max);
	getminmax3d(0, ndata, 0, ntx, 0, nrx, 0, s_i, &s2min, &s2max);
	const float smin = (s1min < s2min) ? s1min : s2min;
	const float smax = (s1max > s2max) ? s1max : s2max;
	//printf("%f %f\n", smin, smax);

	// 1D配列化+1バイト化
	const int ns = ndata * 2 * ntx * nrx;
	unsigned char *s1d = (unsigned char *)malloc(ns * sizeof(unsigned char));
	int is = 0;
	for (int idata = 0; idata < ndata; idata++) {
		for (int i = 0; i < ntx; i++) {
		for (int j = 0; j < nrx; j++) {
			s1d[is++] = (unsigned char)(255 * (s_r[idata][i][j] - smin) / (smax - smin));
		}
		}
		for (int i = 0; i < ntx; i++) {
		for (int j = 0; j < nrx; j++) {
			s1d[is++] = (unsigned char)(255 * (s_i[idata][i][j] - smin) / (smax - smin));
		}
		}
	}
	assert(is == ns);
*/
	// 1D配列化
	const int ns = ndata * 2 * ntx * nrx;
	float *s1d = (float *)malloc(ns * sizeof(float));
	int is = 0;
	for (int idata = 0; idata < ndata; idata++) {
		for (int i = 0; i < ntx; i++) {
		for (int j = 0; j < nrx; j++) {
			s1d[is++] = s_r[idata][i][j];
		}
		}
		for (int i = 0; i < ntx; i++) {
		for (int j = 0; j < nrx; j++) {
			s1d[is++] = s_i[idata][i][j];
		}
		}
	}
	assert(is == ns);

	// 整数データ(Mz=1)
	int idata[6];
	idata[0] = ndata;
	idata[1] = i1 - i0;
	idata[2] = j1 - j0;
	idata[3] = 1;
	idata[4] = ntx;
	idata[5] = nrx;

	// 実数データ
	float fdata[2];
	fdata[0] = dmin;
	fdata[1] = dmax;
	//fdata[2] = smin;
	//fdata[3] = smax;

	// ファイル出力
	FILE *fp;
	if ((fp = fopen(outfile, "wb")) == NULL) {
		printf("%s open error\n", outfile);
		return;
	}

	fwrite(idata, sizeof(int),            6, fp);
	fwrite(fdata, sizeof(float),          2, fp);
	fwrite(d1d,   sizeof(unsigned char), nd, fp);
	fwrite(s1d,   sizeof(float),         ns, fp);

	fclose(fp);

	free(d1d);
	free(s1d);
}
