/*
plot2d.c
2D図形出力, debug用
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <assert.h>
#include "../fdtd2d/ev.h"

extern void getminmax4d(int, int, int, int, int, float ****, float *, float *);
extern void plot2d_contour(float, float, int, int, float **, float, float, int, int);

// 誘電率分布, 送受信点
void plot2d_g(int ndata, int nstep, int nx, int ny, int nz, int nout,
	float ****d, int color, int grid,
	int ntx, int *itx, int *jtx, int *ktx,
	int nrx, int *irx, int *jrx, int *krx, const char fn[])
{
	char str[BUFSIZ], fmt[BUFSIZ];
	const float dm = 0.3f;
	assert((nx > 0) && (ny > 0) && (nz > 0));
	if (ndata <= 0) return;

	// XY配列
	float **f = (float **)malloc(nx * sizeof(float *));
	for (int i = 0; i < nx; i++) {
		f[i] = (float *)malloc(ny * sizeof(float));
	}

	// 最小・最大
	float dmin, dmax;
	getminmax4d(ndata, nx, ny, nz, 0, d, &dmin, &dmax);
	//printf("%f %f\n", dmin, dmax);

	// 開始
	const float s = 2;
	int my = (int)sqrt(nz) - 1;
	if (my < 1) my = 1;
	const int mx = nz / my + 1;
	//printf("%d %d %d\n", nz, mx, my);
	const float w = (mx * nx) + ((mx + 1) * s);
	const float h = (my * ny) + ((my + 1) * s);
	ev2d_init(w, h);

	// 1データ=1ページ
	for (int idata = 0; idata < ndata; idata += nstep) {
		ev2d_newPage();

		// 等高線図, 全Z面の集合
		int k = 0;
		for (int px = 0; px < mx; px++) {
		for (int py = 0; py < my; py++) {
			if (k < nz) {
				// 原点
				const float x0 = (px * nx) + ((px + 1) * s);
				const float y0 = (py * ny) + ((py + 1) * s);

				// XYデータ
				for (int i = 0; i < nx; i++) {
				for (int j = 0; j < ny; j++) {
					f[i][j] = d[idata][i][j][k];
				}
				}

				// 等高線図
				plot2d_contour(x0, y0, nx, ny, f, dmin, dmax, color, grid);

				// 内部領域
				ev2d_setColor(160, 160, 160);
				ev2d_drawRectangle(x0 + nout, y0 + nout, x0 + nx - nout, y0 + ny - nout);

				// データ番号
				ev2d_setColor(0, 0, 0);
				sprintf(str, "%d", k + 1);
				ev2d_drawString(x0, y0 + ny - h / 60, h / 60, str);

				// 送信点
				ev2d_setColor(255, 0, 0);
				for (int n = 0; n < ntx; n++) {
					if (k == ktx[n]) {
						ev2d_fillRectangle(x0 + itx[n] - dm, y0 + jtx[n] - dm, x0 + itx[n] + dm, y0 + jtx[n] + dm);
					}
				}

				// 受信点
				ev2d_setColor(0, 255, 0);
				for (int n = 0; n < nrx; n++) {
					if (k == krx[n]) {
						ev2d_fillRectangle(x0 + irx[n] - dm, y0 + jrx[n] - dm, x0 + irx[n] + dm, y0 + jrx[n] + dm);
					}
				}

				// 図++
				k++;
			}
		}
		}

		// 最小・最大
		strcpy(fmt, "data=%d/%d Nx=%d Ny=%d Nz=%d Mx=%d My=%d Mz=%d Tx=%d Rx=%d min=%.4f max=%.4f");
		sprintf(str, fmt, idata + 1, ndata, nx, ny, nz, nx - 2 * nout, ny - 2 * nout, nz - 2 * nout, ntx, nrx, dmin, dmax);
		ev2d_setColor(0, 0, 0);
		ev2d_drawString(s, s, h / 40, str);

	}
	free(f);

	// 出力
	ev2d_file(1, fn);
	ev2d_output();
}


// 電界分布
void plot2d_e(int ntx, int nstep, int nx, int ny, int nz, int nout,
	float ****d, int db, int grid, int nrect, int **rect, const char fn[])
{
	char str[BUFSIZ], fmt[BUFSIZ];
	const float eps = 1e-12f;
	assert((ntx > 0) && (nstep > 0) && (nx > 0) && (ny > 0) && (nz > 0));

	// 最小・最大(ZXYに注意)
	float dmin, dmax;
	getminmax4d(ntx, nz, nx, ny, db, d, &dmin, &dmax);
	//printf("%f %f\n", dmin, dmax);

	// 開始
	const float s = 2;
	int my = (int)sqrt(nz) - 1;
	if (my < 1) my = 1;
	const int mx = nz / my + 1;
	//printf("%d %d %d\n", nz, mx, my);
	const float w = (mx * nx) + ((mx + 1) * s);
	const float h = (my * ny) + ((my + 1) * s);
	ev2d_init(w, h);

	// 1送信点=1ページ
	for (int itx = 0; itx < ntx; itx += nstep) {
		// 改ページ
		ev2d_newPage();

		// 等高線図, 全Z面の集合
		int k = 0;
		for (int px = 0; px < mx; px++) {
		for (int py = 0; py < my; py++) {
			if (k < nz) {
				// 原点
				const float x0 = (px * nx) + ((px + 1) * s);
				const float y0 = (py * ny) + ((py + 1) * s);

				// XY配列
				float **f = d[itx][k];

				// dB
				if (db) {
					for (int i = 0; i < nx; i++) {
					for (int j = 0; j < ny; j++) {
						f[i][j] = 20 * (float)log10(fmax(f[i][j], eps));
					}
					}
				}

				// 等高線図(カラー)
				plot2d_contour(x0, y0, nx, ny, f, dmin, dmax, 1, grid);

				// 内部領域
				ev2d_setColor(160, 160, 160);
				ev2d_drawRectangle(x0 + nout, y0 + nout, x0 + nx - nout, y0 + ny - nout);

				// 直方体
				ev2d_setColor(0, 0, 0);
				const int idata = 0;  // 図形出力はデータ0のみ
				for (int irect = 0; irect < nrect; irect++) {
					int *ptr = &rect[idata][6 * irect];
					if ((k >= ptr[2]) && (k < ptr[5])) {
						ev2d_drawRectangle(x0 + ptr[0], y0 + ptr[1], x0 + ptr[3], y0 + ptr[4]);
					}
				}

				// データ番号
				ev2d_setColor(0, 0, 0);
				sprintf(str, "%d", k + 1);
				ev2d_drawString(x0, y0 + ny - h / 60, h / 60, str);

				// 図++
				k++;
			}
		}
		}

		// 最小・最大
		strcpy(fmt, db ? "Tx=%d/%d Nx=%d Ny=%d Nz=%d min=%.3fdB max=%.3fdB"
		               : "Tx=%d/%d Nx=%d Ny=%d Nz=%d min=%.6f max=%.6f");
		sprintf(str, fmt, itx + 1, ntx, nx, ny, nz, dmin, dmax);
		ev2d_setColor(0, 0, 0);
		ev2d_drawString(s, s, h / 40, str);

	}

	// 出力
	ev2d_file(1, fn);
	ev2d_output();
}
