/*
pml.c
PML-ABC
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

extern void pmlfactor(int lpml, float *f);

extern float C;

// Ex
void pmlEx(int lpml, int Nx, int Ny, int Nz, float Dt, float Eps0,
	float *Yn, float *Zn, float *dYn, float *dZn,
	float ***Ex, float ***Hy, float ***Hz, float ***Exy, float ***Exz)
{
	float *f = (float *)malloc((2 * lpml + 1) * sizeof(float));
	pmlfactor(lpml, f);

	float *fexy = (float *)malloc((Ny + 1 + 2 * lpml) * sizeof(float));
	float *fexz = (float *)malloc((Nz + 1 + 2 * lpml) * sizeof(float));
	const float cdt = C * Dt;
	float fc = 0;
	for (int j = -lpml; j < Ny + lpml + 1; j++) {
		if      (j <      1) fc = f[- 2 * (j     )    ] / (Yn[1 ] - Yn[0     ]);
		else if (j > Ny - 1) fc = f[+ 2 * (j - Ny)    ] / (Yn[Ny] - Yn[Ny - 1]);
		else                 fc = 0;
		fexy[j + lpml] = 1 / (1 + cdt * fc);
	}
	for (int k = -lpml; k < Nz + lpml + 1; k++) {
		if      (k <      1) fc = f[- 2 * (k     )    ] / (Zn[1 ] - Zn[0     ]);
		else if (k > Nz - 1) fc = f[+ 2 * (k - Nz)    ] / (Zn[Nz] - Zn[Nz - 1]);
		else                 fc = 0;
		fexz[k + lpml] = 1 / (1 + cdt * fc);
	}

	const float r_eps = 1 / Eps0;
	for (int i = -lpml + 0; i < Nx + lpml + 0; i++) {
	for (int j = -lpml + 1; j < Ny + lpml + 0; j++) {
	for (int k = -lpml + 1; k < Nz + lpml + 0; k++) {
		if ((i < 0) || (Nx - 1 < i) ||
		    (j < 0) || (Ny - 0 < j) ||
		    (k < 0) || (Nz - 0 < k)) {
			const float dy = dYn[(j < 0) ? 0 : (j > Ny - 0) ? Ny - 0 : j] * r_eps;
			const float dhz = Hz[i     + lpml][j     + lpml][k     + lpml]
			                - Hz[i     + lpml][j - 1 + lpml][k     + lpml];
			Exy[i + lpml][j + lpml][k + lpml] = (
			Exy[i + lpml][j + lpml][k + lpml] + dy * dhz) * fexy[j + lpml];

			const float dz = dZn[(k < 0) ? 0 : (k > Nz - 0) ? Nz - 0 : k] * r_eps;
			const float dhy = Hy[i     + lpml][j     + lpml][k     + lpml]
			                - Hy[i     + lpml][j     + lpml][k - 1 + lpml];
			Exz[i + lpml][j + lpml][k + lpml] = (
			Exz[i + lpml][j + lpml][k + lpml] - dz * dhy) * fexz[k + lpml];

			Ex[i + lpml][j + lpml][k + lpml] = Exy[i + lpml][j + lpml][k + lpml]
			                                 + Exz[i + lpml][j + lpml][k + lpml];
		}
	}
	}
	}

	free(fexy);
	free(fexz);
	free(f);
}


// Ey
void pmlEy(int lpml, int Nx, int Ny, int Nz, float Dt, float Eps0,
	float *Zn, float *Xn, float *dZn, float *dXn,
	float ***Ey, float ***Hz, float ***Hx, float ***Eyz, float ***Eyx)
{
	float *f = (float *)malloc((2 * lpml + 1) * sizeof(float));
	pmlfactor(lpml, f);

	float *feyz = (float *)malloc((Nz + 1 + 2 * lpml) * sizeof(float));
	float *feyx = (float *)malloc((Nx + 1 + 2 * lpml) * sizeof(float));
	const float cdt = C * Dt;
	float fc = 0;
	for (int k = -lpml; k < Nz + lpml + 1; k++) {
		if      (k <      1) fc = f[- 2 * (k     )    ] / (Zn[1 ] - Zn[0     ]);
		else if (k > Nz - 1) fc = f[+ 2 * (k - Nz)    ] / (Zn[Nz] - Zn[Nz - 1]);
		else                 fc = 0;
		feyz[k + lpml] = 1 / (1 + cdt * fc);
	}
	for (int i = -lpml; i < Nx + lpml + 1; i++) {
		if      (i <      1) fc = f[- 2 * (i     )    ] / (Xn[1 ] - Xn[0     ]);
		else if (i > Nx - 1) fc = f[+ 2 * (i - Nx)    ] / (Xn[Nx] - Xn[Nx - 1]);
		else                 fc = 0;
		feyx[i + lpml] = 1 / (1 + cdt * fc);
	}

	const float r_eps = 1 / Eps0;
	for (int i = -lpml + 1; i < Nx + lpml + 0; i++) {
	for (int j = -lpml + 0; j < Ny + lpml + 0; j++) {
	for (int k = -lpml + 1; k < Nz + lpml + 0; k++) {
		if ((i < 0) || (Nx - 0 < i) ||
		    (j < 0) || (Ny - 1 < j) ||
		    (k < 0) || (Nz - 0 < k)) {
			const float dz = dZn[(k < 0) ? 0 : (k > Nz - 0) ? Nz - 0 : k] * r_eps;
			const float dhx = Hx[i     + lpml][j     + lpml][k     + lpml]
			                - Hx[i     + lpml][j     + lpml][k - 1 + lpml];
			Eyz[i + lpml][j + lpml][k + lpml] = (
			Eyz[i + lpml][j + lpml][k + lpml] + dz * dhx) * feyz[k + lpml];

			const float dx = dXn[(i < 0) ? 0 : (i > Nx - 0) ? Nx - 0 : i] * r_eps;
			const float dhz = Hz[i     + lpml][j     + lpml][k     + lpml]
			                - Hz[i - 1 + lpml][j     + lpml][k     + lpml];
			Eyx[i + lpml][j + lpml][k + lpml] = (
			Eyx[i + lpml][j + lpml][k + lpml] - dx * dhz) * feyx[i + lpml];

			Ey[i + lpml][j + lpml][k + lpml] = Eyz[i + lpml][j + lpml][k + lpml]
			                                 + Eyx[i + lpml][j + lpml][k + lpml];
		}
	}
	}
	}

	free(feyz);
	free(feyx);
	free(f);
}


// Ez
void pmlEz(int lpml, int Nx, int Ny, int Nz, float Dt, float Eps0,
	float *Xn, float *Yn, float *dXn, float *dYn,
	float ***Ez, float ***Hx, float ***Hy, float ***Ezx, float ***Ezy)
{
	float *f = (float *)malloc((2 * lpml + 1) * sizeof(float));
	pmlfactor(lpml, f);

	float *fezx = (float *)malloc((Nx + 1 + 2 * lpml) * sizeof(float));
	float *fezy = (float *)malloc((Ny + 1 + 2 * lpml) * sizeof(float));
	const float cdt = C * Dt;
	float fc = 0;
	for (int i = -lpml; i < Nx + lpml + 1; i++) {
		if      (i <      1) fc = f[- 2 * (i     )    ] / (Xn[1 ] - Xn[0     ]);
		else if (i > Nx - 1) fc = f[+ 2 * (i - Nx)    ] / (Xn[Nx] - Xn[Nx - 1]);
		else                 fc = 0;
		fezx[i + lpml] = 1 / (1 + cdt * fc);
	}
	for (int j = -lpml; j < Ny + lpml + 1; j++) {
		if      (j <      1) fc = f[- 2 * (j     )    ] / (Yn[1 ] - Yn[0     ]);
		else if (j > Ny - 1) fc = f[+ 2 * (j - Ny)    ] / (Yn[Ny] - Yn[Ny - 1]);
		else                 fc = 0;
		fezy[j + lpml] = 1 / (1 + cdt * fc);
	}
	//for (int i = 0; i < Nx + 1 + 2 * lpml; i++) printf("%d %e\n", i, fezx[i]);
	//for (int j = 0; j < Ny + 1 + 2 * lpml; j++) printf("%d %e\n", j, fezy[j]);

	const float r_eps = 1 / Eps0;
	for (int i = -lpml + 1; i < Nx + lpml + 0; i++) {
	for (int j = -lpml + 1; j < Ny + lpml + 0; j++) {
	for (int k = -lpml + 0; k < Nz + lpml + 0; k++) {
		if ((i < 0) || (Nx - 0 < i) ||
		    (j < 0) || (Ny - 0 < j) ||
		    (k < 0) || (Nz - 1 < k)) {
			const float dx = dXn[(i < 0) ? 0 : (i > Nx - 0) ? Nx - 0 : i] * r_eps;
			const float dhy = Hy[i     + lpml][j     + lpml][k     + lpml]
			                - Hy[i - 1 + lpml][j     + lpml][k     + lpml];
			Ezx[i + lpml][j + lpml][k + lpml] = (
			Ezx[i + lpml][j + lpml][k + lpml] + dx * dhy) * fezx[i + lpml];

			const float dy = dYn[(j < 0) ? 0 : (j > Ny - 0) ? Ny - 0 : j] * r_eps;
			const float dhx = Hx[i     + lpml][j     + lpml][k     + lpml]
			                - Hx[i     + lpml][j - 1 + lpml][k     + lpml];
			Ezy[i + lpml][j + lpml][k + lpml] = (
			Ezy[i + lpml][j + lpml][k + lpml] - dy * dhx) * fezy[j + lpml];

			Ez[i + lpml][j + lpml][k + lpml] = Ezx[i + lpml][j + lpml][k + lpml]
			                                 + Ezy[i + lpml][j + lpml][k + lpml];
		}
	}
	}
	}

	free(fezx);
	free(fezy);
	free(f);
}


// Hx
void pmlHx(int lpml, int Nx, int Ny, int Nz, float Dt,
	float *Yn, float *Zn, float *dYc, float *dZc,
	float ***Hx, float ***Ey, float ***Ez, float ***Hxy, float ***Hxz)
{
	float *f = (float *)malloc((2 * lpml + 1) * sizeof(float));
	pmlfactor(lpml, f);

	float *fhxy = (float *)malloc((Ny + 0 + 2 * lpml) * sizeof(float));
	float *fhxz = (float *)malloc((Nz + 0 + 2 * lpml) * sizeof(float));
	const float cdt = C * Dt;
	float fc = 0;
	for (int j = -lpml; j < Ny + lpml + 0; j++) {
		if      (j <      0) fc = f[- 2 * (j     ) - 1] / (Yn[1 ] - Yn[0     ]);
		else if (j > Ny - 1) fc = f[+ 2 * (j - Ny) + 1] / (Yn[Ny] - Yn[Ny - 1]);
		else                 fc = 0;
		fhxy[j + lpml] = 1 / (1 + cdt * fc);
	}
	for (int k = -lpml; k < Nz + lpml + 0; k++) {
		if      (k <      0) fc = f[- 2 * (k     ) - 1] / (Zn[1 ] - Zn[0     ]);
		else if (k > Nz - 1) fc = f[+ 2 * (k - Nz) + 1] / (Zn[Nz] - Zn[Nz - 1]);
		else                 fc = 0;
		fhxz[k + lpml] = 1 / (1 + cdt * fc);
	}

	const float r_amu = 1;  // 磁性体なし
	for (int i = -lpml + 1; i < Nx + lpml + 0; i++) {
	for (int j = -lpml + 0; j < Ny + lpml + 0; j++) {
	for (int k = -lpml + 0; k < Nz + lpml + 0; k++) {
		if ((i < 0) || (Nx - 0 < i) ||
		    (j < 0) || (Ny - 1 < j) ||
		    (k < 0) || (Nz - 1 < k)) {
			const float dy = dYc[(j < 0) ? 0 : (j > Ny - 1) ? Ny - 1 : j] * r_amu;
			const float dez = Ez[i     + lpml][j + 1 + lpml][k     + lpml]
			                - Ez[i     + lpml][j     + lpml][k     + lpml];
			Hxy[i + lpml][j + lpml][k + lpml] = (
			Hxy[i + lpml][j + lpml][k + lpml] - (dy * dez)) * fhxy[j + lpml];

			const float dz = dZc[(k < 0) ? 0 : (k > Nz - 1) ? Nz - 1 : k] * r_amu;
			const float dey = Ey[i     + lpml][j     + lpml][k + 1 + lpml]
			                - Ey[i     + lpml][j     + lpml][k     + lpml];
			Hxz[i + lpml][j + lpml][k + lpml] = (
			Hxz[i + lpml][j + lpml][k + lpml] + (dz * dey)) * fhxz[k + lpml];

			Hx[i + lpml][j + lpml][k + lpml] = Hxy[i + lpml][j + lpml][k + lpml]
		                                     + Hxz[i + lpml][j + lpml][k + lpml];
		}
	}
	}
	}

	free(fhxy);
	free(fhxz);
	free(f);
}


// Hy
void pmlHy(int lpml, int Nx, int Ny, int Nz, float Dt,
	float *Zn, float *Xn, float *dZc, float *dXc,
	float ***Hy, float ***Ez, float ***Ex, float ***Hyz, float ***Hyx)
{
	float *f = (float *)malloc((2 * lpml + 1) * sizeof(float));
	pmlfactor(lpml, f);

	float *fhyz = (float *)malloc((Nz + 0 + 2 * lpml) * sizeof(float));
	float *fhyx = (float *)malloc((Nx + 0 + 2 * lpml) * sizeof(float));
	const float cdt = C * Dt;
	float fc = 0;
	for (int k = -lpml; k < Nz + lpml + 0; k++) {
		if      (k <      0) fc = f[- 2 * (k     ) - 1] / (Zn[1 ] - Zn[0     ]);
		else if (k > Nz - 1) fc = f[+ 2 * (k - Nz) + 1] / (Zn[Nz] - Zn[Nz - 1]);
		else                 fc = 0;
		fhyz[k + lpml] = 1 / (1 + cdt * fc);
	}
	for (int i = -lpml; i < Nx + lpml + 0; i++) {
		if      (i <      0) fc = f[- 2 * (i     ) - 1] / (Xn[1 ] - Xn[0     ]);
		else if (i > Nx - 1) fc = f[+ 2 * (i - Nx) + 1] / (Xn[Nx] - Xn[Nx - 1]);
		else                 fc = 0;
		fhyx[i + lpml] = 1 / (1 + cdt * fc);
	}

	const float r_amu = 1;  // 磁性体なし
	for (int i = -lpml + 0; i < Nx + lpml + 0; i++) {
	for (int j = -lpml + 1; j < Ny + lpml + 0; j++) {
	for (int k = -lpml + 0; k < Nz + lpml + 0; k++) {
		if ((i < 0) || (Nx - 1 < i) ||
		    (j < 0) || (Ny - 0 < j) ||
		    (k < 0) || (Nz - 1 < k)) {
			const float dz = dZc[(k < 0) ? 0 : (k > Nz - 1) ? Nz - 1 : k] * r_amu;
			const float dex = Ex[i     + lpml][j     + lpml][k + 1 + lpml]
			                - Ex[i     + lpml][j     + lpml][k     + lpml];
			Hyz[i + lpml][j + lpml][k + lpml] = (
			Hyz[i + lpml][j + lpml][k + lpml] - (dz * dex)) * fhyz[k + lpml];

			const float dx = dXc[(i < 0) ? 0 : (i > Nx - 1) ? Nx - 1 : i] * r_amu;
			const float dez = Ez[i + 1 + lpml][j     + lpml][k     + lpml]
			                - Ez[i     + lpml][j     + lpml][k     + lpml];
			Hyx[i + lpml][j + lpml][k + lpml] = (
			Hyx[i + lpml][j + lpml][k + lpml] + (dx * dez)) * fhyx[i + lpml];

			Hy[i + lpml][j + lpml][k + lpml] = Hyz[i + lpml][j + lpml][k + lpml]
		                                     + Hyx[i + lpml][j + lpml][k + lpml];
		}
	}
	}
	}

	free(fhyz);
	free(fhyx);
	free(f);
}


// Hz
void pmlHz(int lpml, int Nx, int Ny, int Nz, float Dt,
	float *Xn, float *Yn, float *dXc, float *dYc,
	float ***Hz, float ***Ex, float ***Ey, float ***Hzx, float ***Hzy)
{
	float *f = (float *)malloc((2 * lpml + 1) * sizeof(float));
	pmlfactor(lpml, f);

	float *fhzx = (float *)malloc((Nx + 0 + 2 * lpml) * sizeof(float));
	float *fhzy = (float *)malloc((Ny + 0 + 2 * lpml) * sizeof(float));
	const float cdt = C * Dt;
	float fc = 0;
	for (int i = -lpml; i < Nx + lpml + 0; i++) {
		if      (i <      0) fc = f[- 2 * (i     ) - 1] / (Xn[1 ] - Xn[0     ]);
		else if (i > Nx - 1) fc = f[+ 2 * (i - Nx) + 1] / (Xn[Nx] - Xn[Nx - 1]);
		else                 fc = 0;
		fhzx[i + lpml] = 1 / (1 + cdt * fc);
	}
	for (int j = -lpml; j < Ny + lpml + 0; j++) {
		if      (j <      0) fc = f[- 2 * (j     ) - 1] / (Yn[1 ] - Yn[0     ]);
		else if (j > Ny - 1) fc = f[+ 2 * (j - Ny) + 1] / (Yn[Ny] - Yn[Ny - 1]);
		else                 fc = 0;
		fhzy[j + lpml] = 1 / (1 + cdt * fc);
	}

	const float r_amu = 1;  // 磁性体なし
	for (int i = -lpml + 0; i < Nx + lpml + 0; i++) {
	for (int j = -lpml + 0; j < Ny + lpml + 0; j++) {
	for (int k = -lpml + 1; k < Nz + lpml + 0; k++) {
		if ((i < 0) || (Nx - 1 < i) ||
		    (j < 0) || (Ny - 1 < j) ||
		    (k < 0) || (Nz - 0 < k)) {
			const float dx = dXc[(i < 0) ? 0 : (i > Nx - 1) ? Nx - 1 : i] * r_amu;
			const float dey = Ey[i + 1 + lpml][j     + lpml][k     + lpml]
			                - Ey[i     + lpml][j     + lpml][k     + lpml];
			Hzx[i + lpml][j + lpml][k + lpml] = (
			Hzx[i + lpml][j + lpml][k + lpml] - (dx * dey)) * fhzx[i + lpml];

			const float dy = dYc[(j < 0) ? 0 : (j > Ny - 1) ? Ny - 1 : j] * r_amu;
			const float dex = Ex[i     + lpml][j + 1 + lpml][k     + lpml]
			                - Ex[i     + lpml][j     + lpml][k     + lpml];
			Hzy[i + lpml][j + lpml][k + lpml] = (
			Hzy[i + lpml][j + lpml][k + lpml] + (dy * dex)) * fhzy[j + lpml];

			Hz[i + lpml][j + lpml][k + lpml] = Hzx[i + lpml][j + lpml][k + lpml]
		                                     + Hzy[i + lpml][j + lpml][k + lpml];
		}
	}
	}
	}

	free(fhzx);
	free(fhzy);
	free(f);
}
