/*
solve.c
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <assert.h>

static void setupFactor(int, int, int, float, float ***, float ***, float, float, float ***, float ***, float ***, float ***, float ***, float ***);
static void initfield(int, int, int, int, int, float ***, float ***, float ***, float ***, float ***, float ***, float *, float *, float ***, float ***, float ***, float ***, float ***, float ***, float ***, float ***, float ***, float ***, float ***, float ***, float ***, float ***, float ***, float ***, float ***, float ***, int, float ***, float ***, float ***, float ***, float ***, float ***);
static void dft(int, int, int, int, int, int *, int *, int *, float ***, float ***, float ***, float, float, float *, float *, int, float ***, float ***, float ***, float ***, float ***, float ***);
static void average(int, int, int, int, float ***, float ***, float ***, float ***, float ***, float ***, float []);

extern void dftfactor(int, float, float, float *, float *);
extern void updateEx(int, int, int, int, float ***, float ***, float ***, float *, float *, float ***, float ***);
extern void updateEy(int, int, int, int, float ***, float ***, float ***, float *, float *, float ***, float ***);
extern void updateEz(int, int, int, int, float ***, float ***, float ***, float *, float *, float ***, float ***);
extern void updateHx(int, int, int, int, float ***, float ***, float ***, float *, float *);
extern void updateHy(int, int, int, int, float ***, float ***, float ***, float *, float *);
extern void updateHz(int, int, int, int, float ***, float ***, float ***, float *, float *);
extern void murEx(int, int, int, float, float *, float *, float ***, float ***, float ***, float);
extern void murEy(int, int, int, float, float *, float *, float ***, float ***, float ***, float);
extern void murEz(int, int, int, float, float *, float *, float ***, float ***, float ***, float);
extern void pmlEx(int, int, int, int, float, float, float *, float *, float *, float *, float ***, float ***, float ***, float ***, float ***);
extern void pmlEy(int, int, int, int, float, float, float *, float *, float *, float *, float ***, float ***, float ***, float ***, float ***);
extern void pmlEz(int, int, int, int, float, float, float *, float *, float *, float *, float ***, float ***, float ***, float ***, float ***);
extern void pmlHx(int, int, int, int, float, float *, float *, float *, float *, float ***, float ***, float ***, float ***, float ***);
extern void pmlHy(int, int, int, int, float, float *, float *, float *, float *, float ***, float ***, float ***, float ***, float ***);
extern void pmlHz(int, int, int, int, float, float *, float *, float *, float *, float ***, float ***, float ***, float ***, float ***);
extern float vfeed(float, float);

extern float C;

void solve(
	int Niter, int Nx, int Ny, int Nz, float Freq, float Dt, int ABC,
	int iTx, int jTx, int kTx, int NRx, int *iRx, int *jRx, int *kRx,
	float *Xn, float *Yn, float *Zn, float *dXn, float *dYn, float *dZn, float *dXc, float *dYc, float *dZc,
	float ***epsr, float ***sigm, float Eps0, float Sig0,
	float ***Ex, float ***Ey, float ***Ez, float ***Hx, float ***Hy, float ***Hz,
	float ***C1Ex, float ***C2Ex, float ***C1Ey, float ***C2Ey, float ***C1Ez, float ***C2Ez,
	float ***Exmury, float ***Exmurz, float ***Eymurz, float ***Eymurx, float ***Ezmurx, float ***Ezmury,
	float ***Exy, float ***Exz, float ***Eyz, float ***Eyx, float ***Ezx, float ***Ezy,
	float ***Hxy, float ***Hxz, float ***Hyz, float ***Hyx, float ***Hzx, float ***Hzy,
	float *S_r, float *S_i,
	int ilog, int iout, float ***Ex_r, float ***Ex_i, float ***Ey_r, float ***Ey_i, float ***Ez_r, float ***Ez_i, float ***Ea)
{
	//printf("%d %d %d %d %e %e %d\n", Niter, Nx, Ny, Nz, Freq, Dt, ABC);
	//printf("%d %d %d %d %d %d %d\n", iTx, jTx, kTx, NRx, iRx[0], jRx[0], kRx[0]);

	// DFT係数
	float *fcos = (float *)malloc((Niter + 1) * sizeof(float));
	float *fsin = (float *)malloc((Niter + 1) * sizeof(float));
	dftfactor(Niter, Freq, Dt, fcos, fsin);

	// FDTD係数
	setupFactor(Nx, Ny, Nz, Dt,
		epsr, sigm, Eps0, Sig0,
		C1Ex, C2Ex, C1Ey, C2Ey, C1Ez, C2Ez);

	// 電磁界初期化
	initfield(ABC, Nx, Ny, Nz, NRx,
		Ex, Ey, Ez, Hx, Hy, Hz,
		S_r, S_i,
		Exmury, Exmurz, Eymurz, Eymurx, Ezmurx, Ezmury,
		Exy, Exz, Eyz, Eyx, Ezx, Ezy,
		Hxy, Hxz, Hyz, Hyx, Hzx, Hzy,
		iout, Ex_r, Ex_i, Ey_r, Ey_i, Ez_r, Ez_i);

	// FDTD反復計算
	float t = 0;
	for (int iter = 0; iter <= Niter; iter++) {
		// E更新
		updateEx(ABC, Nx, Ny, Nz, Ex, Hy, Hz, dYn, dZn, C1Ex, C2Ex);
		updateEy(ABC, Nx, Ny, Nz, Ey, Hz, Hx, dZn, dXn, C1Ey, C2Ey);
		updateEz(ABC, Nx, Ny, Nz, Ez, Hx, Hy, dXn, dYn, C1Ez, C2Ez);

		// 給電点(Z方向)
		Ez[iTx + ABC][jTx + ABC][kTx + ABC] = vfeed(t, Freq) / (Zn[kTx + 1] - Zn[kTx]);

		// ABC
		if (ABC > 0) {
			pmlEx(ABC, Nx, Ny, Nz, Dt, Eps0, Yn, Zn, dYn, dZn, Ex, Hy, Hz, Exy, Exz);
			pmlEy(ABC, Nx, Ny, Nz, Dt, Eps0, Zn, Xn, dZn, dXn, Ey, Hz, Hx, Eyz, Eyx);
			pmlEz(ABC, Nx, Ny, Nz, Dt, Eps0, Xn, Yn, dXn, dYn, Ez, Hx, Hy, Ezx, Ezy);
		}
		else {
			murEx(Nx, Ny, Nz, Dt, Yn, Zn, Ex, Exmury, Exmurz, Eps0);
			murEy(Nx, Ny, Nz, Dt, Zn, Xn, Ey, Eymurz, Eymurx, Eps0);
			murEz(Nx, Ny, Nz, Dt, Xn, Yn, Ez, Ezmurx, Ezmury, Eps0);
		}

		// H更新
		updateHx(ABC, Nx, Ny, Nz, Hx, Ey, Ez, dYc, dZc);
		updateHy(ABC, Nx, Ny, Nz, Hy, Ez, Ex, dZc, dXc);
		updateHz(ABC, Nx, Ny, Nz, Hz, Ex, Ey, dXc, dYc);

		// ABC
		if (ABC > 0) {
			pmlHx(ABC, Nx, Ny, Nz, Dt, Yn, Zn, dYc, dZc, Hx, Ey, Ez, Hxy, Hxz);
			pmlHy(ABC, Nx, Ny, Nz, Dt, Zn, Xn, dZc, dXc, Hy, Ez, Ex, Hyz, Hyx);
			pmlHz(ABC, Nx, Ny, Nz, Dt, Xn, Yn, dXc, dYc, Hz, Ex, Ey, Hzx, Hzy);
		}

		// DFT
		dft(ABC, Nx, Ny, Nz, NRx, iRx, jRx, kRx, Ex, Ey, Ez, fcos[iter], fsin[iter], S_r, S_i, iout, Ex_r, Ex_i, Ey_r, Ey_i, Ez_r, Ez_i);

		// 時間ステップ
		t += Dt;

		// debug, 平均電磁界出力
		if (ilog && (iter % 50 == 0)) {
			float fsum[2];
			average(ABC, Nx, Ny, Nz, Ex, Ey, Ez, Hx, Hy, Hz, fsum);
			printf("%5d %.6f %.6f\n", iter, fsum[0], fsum[1]);
		}
	}

	// debug, 電界分布, セル中心, データ0=プロセス0=通信不要
	if (iout) {
		for (int i = 0; i < Nx; i++) {
		for (int j = 0; j < Ny; j++) {
		for (int k = 0; k < Nz; k++) {
			// セル平均(ZXYに注意)
			const float ex_r = (Ex_r[i][j][k] + Ex_r[i    ][j + 1][k    ] + Ex_r[i    ][j    ][k + 1] + Ex_r[i    ][j + 1][k + 1]) / 4;
			const float ex_i = (Ex_i[i][j][k] + Ex_i[i    ][j + 1][k    ] + Ex_i[i    ][j    ][k + 1] + Ex_i[i    ][j + 1][k + 1]) / 4;
			const float ey_r = (Ey_r[i][j][k] + Ey_r[i    ][j    ][k + 1] + Ey_r[i + 1][j    ][k    ] + Ey_r[i + 1][j    ][k + 1]) / 4;
			const float ey_i = (Ey_i[i][j][k] + Ey_i[i    ][j    ][k + 1] + Ey_i[i + 1][j    ][k    ] + Ey_i[i + 1][j    ][k + 1]) / 4;
			const float ez_r = (Ez_r[i][j][k] + Ez_r[i + 1][j    ][k    ] + Ez_r[i    ][j + 1][k    ] + Ez_r[i + 1][j + 1][k    ]) / 4;
			const float ez_i = (Ez_i[i][j][k] + Ez_i[i + 1][j    ][k    ] + Ez_i[i    ][j + 1][k    ] + Ez_i[i + 1][j + 1][k    ]) / 4;
			Ea[k][i][j] = (float)sqrt((ex_r * ex_r) + (ex_i * ex_i) + (ey_r * ey_r) + (ey_i * ey_i) + (ez_r * ez_r) + (ez_i * ez_i));
			//printf("%d %d %d %e\n", k, i, j, Ea[k][i][j]);
		}
		}
		}
	}

	// free
	free(fcos);
	free(fsin);
}


// 電磁界初期化
static void initfield(int ABC, int Nx, int Ny, int Nz, int NRx,
	float ***Ex, float ***Ey, float ***Ez, float ***Hx, float ***Hy, float ***Hz,
	float *S_r, float *S_i,
	float ***Exmury, float ***Exmurz, float ***Eymurz, float ***Eymurx, float ***Ezmurx, float ***Ezmury,
	float ***Exy, float ***Exz, float ***Eyz, float ***Eyx, float ***Ezx, float ***Ezy,
	float ***Hxy, float ***Hxz, float ***Hyz, float ***Hyx, float ***Hzx, float ***Hzy,
	int iout, float ***Ex_r, float ***Ex_i, float ***Ey_r, float ***Ey_i, float ***Ez_r, float ***Ez_i)
{
	assert((Nx > 0) && (Ny > 0) && (Nz > 0) && (NRx > 0));

	// E
	for (int i = - ABC; i < Nx + 0 + ABC; i++) {
	for (int j = - ABC; j < Ny + 1 + ABC; j++) {
		memset(Ex[i + ABC][j + ABC], 0, (Nz + 1 + 2 * ABC) * sizeof(float));
	}
	}
	for (int i = - ABC; i < Nx + 1 + ABC; i++) {
	for (int j = - ABC; j < Ny + 0 + ABC; j++) {
		memset(Ey[i + ABC][j + ABC], 0, (Nz + 1 + 2 * ABC) * sizeof(float));
	}
	}
	for (int i = - ABC; i < Nx + 1 + ABC; i++) {
	for (int j = - ABC; j < Ny + 1 + ABC; j++) {
		memset(Ez[i + ABC][j + ABC], 0, (Nz + 0 + 2 * ABC) * sizeof(float));
	}
	}
	// H
	for (int i = - ABC; i < Nx + 1 + ABC; i++) {
	for (int j = - ABC; j < Ny + 0 + ABC; j++) {
		memset(Hx[i + ABC][j + ABC], 0, (Nz + 0 + 2 * ABC) * sizeof(float));
	}
	}
	for (int i = - ABC; i < Nx + 0 + ABC; i++) {
	for (int j = - ABC; j < Ny + 1 + ABC; j++) {
		memset(Hy[i + ABC][j + ABC], 0, (Nz + 0 + 2 * ABC) * sizeof(float));
	}
	}
	for (int i = - ABC; i < Nx + 0 + ABC; i++) {
	for (int j = - ABC; j < Ny + 0 + ABC; j++) {
		memset(Hz[i + ABC][j + ABC], 0, (Nz + 1 + 2 * ABC) * sizeof(float));
	}
	}

	// S行列
	memset(S_r, 0, NRx * sizeof(float));
	memset(S_i, 0, NRx * sizeof(float));

	// ABC
	if (ABC == 0) {
		for (int m = 0; m < 2; m++) {
			for (int k = 0; k < Nz + 1; k++) {
				memset(Exmury[m][k], 0, (Nx + 0) * sizeof(float));
			}
			for (int j = 0; j < Ny + 1; j++) {
				memset(Exmurz[m][j], 0, (Nx + 0) * sizeof(float));
			}
			for (int i = 0; i < Nx + 1; i++) {
				memset(Eymurz[m][i], 0, (Ny + 0) * sizeof(float));
			}
			for (int k = 0; k < Nz + 1; k++) {
				memset(Eymurx[m][k], 0, (Ny + 0) * sizeof(float));
			}
			for (int j = 0; j < Ny + 1; j++) {
				memset(Ezmurx[m][j], 0, (Nz + 0) * sizeof(float));
			}
			for (int i = 0; i < Nx + 1; i++) {
				memset(Ezmury[m][i], 0, (Nz + 0) * sizeof(float));
			}
		}
	}
	else {
		// E
		for (int i = - ABC; i < Nx + 0 + ABC; i++) {
		for (int j = - ABC; j < Ny + 1 + ABC; j++) {
			memset(Exy[i + ABC][j + ABC], 0, (Nz + 1 + 2 * ABC) * sizeof(float));
			memset(Exz[i + ABC][j + ABC], 0, (Nz + 1 + 2 * ABC) * sizeof(float));
		}
		}
		for (int i = - ABC; i < Nx + 1 + ABC; i++) {
		for (int j = - ABC; j < Ny + 0 + ABC; j++) {
			memset(Eyz[i + ABC][j + ABC], 0, (Nz + 1 + 2 * ABC) * sizeof(float));
			memset(Eyx[i + ABC][j + ABC], 0, (Nz + 1 + 2 * ABC) * sizeof(float));
		}
		}
		for (int i = - ABC; i < Nx + 1 + ABC; i++) {
		for (int j = - ABC; j < Ny + 1 + ABC; j++) {
			memset(Ezx[i + ABC][j + ABC], 0, (Nz + 0 + 2 * ABC) * sizeof(float));
			memset(Ezy[i + ABC][j + ABC], 0, (Nz + 0 + 2 * ABC) * sizeof(float));
		}
		}
		// H
		for (int i = - ABC; i < Nx + 1 + ABC; i++) {
		for (int j = - ABC; j < Ny + 0 + ABC; j++) {
			memset(Hxy[i + ABC][j + ABC], 0, (Nz + 0 + 2 * ABC) * sizeof(float));
			memset(Hxz[i + ABC][j + ABC], 0, (Nz + 0 + 2 * ABC) * sizeof(float));
		}
		}
		for (int i = - ABC; i < Nx + 0 + ABC; i++) {
		for (int j = - ABC; j < Ny + 1 + ABC; j++) {
			memset(Hyz[i + ABC][j + ABC], 0, (Nz + 0 + 2 * ABC) * sizeof(float));
			memset(Hyx[i + ABC][j + ABC], 0, (Nz + 0 + 2 * ABC) * sizeof(float));
		}
		}
		for (int i = - ABC; i < Nx + 0 + ABC; i++) {
		for (int j = - ABC; j < Ny + 0 + ABC; j++) {
			memset(Hzx[i + ABC][j + ABC], 0, (Nz + 1 + 2 * ABC) * sizeof(float));
			memset(Hzy[i + ABC][j + ABC], 0, (Nz + 1 + 2 * ABC) * sizeof(float));
		}
		}
	}

	// debug, E
	if (iout) {
		for (int i = 0; i < Nx + 0; i++) {
		for (int j = 0; j < Ny + 1; j++) {
			memset(Ex_r[i][j], 0, (Nz + 1) * sizeof(float));
			memset(Ex_i[i][j], 0, (Nz + 1) * sizeof(float));
		}
		}
		for (int i = 0; i < Nx + 1; i++) {
		for (int j = 0; j < Ny + 0; j++) {
			memset(Ey_r[i][j], 0, (Nz + 1) * sizeof(float));
			memset(Ey_i[i][j], 0, (Nz + 1) * sizeof(float));
		}
		}
		for (int i = 0; i < Nx + 1; i++) {
		for (int j = 0; j < Ny + 1; j++) {
			memset(Ez_r[i][j], 0, (Nz + 0) * sizeof(float));
			memset(Ez_i[i][j], 0, (Nz + 0) * sizeof(float));
		}
		}
	}
}


// FDTD係数
static void setupFactor(int Nx, int Ny, int Nz, float Dt,
	float ***epsr, float ***sigm, float Eps0, float Sig0,
	float ***C1Ex, float ***C2Ex, float ***C1Ey, float ***C2Ey, float ***C1Ez, float ***C2Ez)
{
	const float pi = (float)(4 * atan(1));
	const float eta = C * 4 * pi * 1e-7f;

	float eps, sig;

	// Ex
	for (int i = 0; i < Nx + 0; i++) {
	for (int j = 0; j < Ny + 1; j++) {
	for (int k = 0; k < Nz + 1; k++) {
		if ((j == 0) || (j == Ny) || (k == 0) || (k == Nz)) {
			// 境界
			eps = Eps0;
			sig = Sig0;
		}
		else {
			// 内部:4点の平均
			eps = (epsr[i][j][k] + epsr[i][j - 1][k] + epsr[i][j][k - 1] + epsr[i][j - 1][k - 1]) / 4;
			sig = (sigm[i][j][k] + sigm[i][j - 1][k] + sigm[i][j][k - 1] + sigm[i][j - 1][k - 1]) / 4;
		}
		C2Ex[i][j][k] = 1 / (eps + (sig * eta * C * Dt));
		C1Ex[i][j][k] = C2Ex[i][j][k] * eps;
	}
	}
	}

	// Ey
	for (int i = 0; i < Nx + 1; i++) {
	for (int j = 0; j < Ny + 0; j++) {
	for (int k = 0; k < Nz + 1; k++) {
		if ((k == 0) || (k == Nz) || (i == 0) || (i == Nx)) {
			// 境界
			eps = Eps0;
			sig = Sig0;
		}
		else {
			// 内部:4点の平均
			eps = (epsr[i][j][k] + epsr[i][j][k - 1] + epsr[i - 1][j][k] + epsr[i - 1][j][k - 1]) / 4;
			sig = (sigm[i][j][k] + sigm[i][j][k - 1] + sigm[i - 1][j][k] + sigm[i - 1][j][k - 1]) / 4;
		}
		C2Ey[i][j][k] = 1 / (eps + (sig * eta * C * Dt));
		C1Ey[i][j][k] = C2Ey[i][j][k] * eps;
	}
	}
	}

	// Ez
	for (int i = 0; i < Nx + 1; i++) {
	for (int j = 0; j < Ny + 1; j++) {
	for (int k = 0; k < Nz + 0; k++) {
		if ((i == 0) || (i == Nx) || (j == 0) || (j == Ny)) {
			// 境界
			eps = Eps0;
			sig = Sig0;
		}
		else {
			// 内部:4点の平均
			eps = (epsr[i][j][k] + epsr[i - 1][j][k] + epsr[i][j - 1][k] + epsr[i - 1][j - 1][k]) / 4;
			sig = (sigm[i][j][k] + sigm[i - 1][j][k] + sigm[i][j - 1][k] + sigm[i - 1][j - 1][k]) / 4;
		}
		C2Ez[i][j][k] = 1 / (eps + (sig * eta * C * Dt));
		C1Ez[i][j][k] = C2Ez[i][j][k] * eps;
		//printf("%d %d %d %f %f\n", i, j, k, C1Ex[i][j][k], C2Ez[i][j][k]);
	}
	}
	}
}


// 平均電磁界
static void average(int ABC, int Nx, int Ny, int Nz,
	float ***Ex, float ***Ey, float ***Ez, float ***Hx, float ***Hy, float ***Hz, float fsum[])
{
	// <E>
	float sume = 0;
	int nume = 0;
	for (int i = 0; i < Nx + 0; i++) {
	for (int j = 0; j < Ny + 1; j++) {
	for (int k = 0; k < Nz + 1; k++) {
		sume += (float)fabs(Ex[i + ABC][j + ABC][k + ABC]);
		nume++;
	}
	}
	}
	for (int i = 0; i < Nx + 1; i++) {
	for (int j = 0; j < Ny + 0; j++) {
	for (int k = 0; k < Nz + 1; k++) {
		sume += (float)fabs(Ey[i + ABC][j + ABC][k + ABC]);
		nume++;
	}
	}
	}
	for (int i = 0; i < Nx + 1; i++) {
	for (int j = 0; j < Ny + 1; j++) {
	for (int k = 0; k < Nz + 0; k++) {
		sume += (float)fabs(Ez[i + ABC][j + ABC][k + ABC]);
		nume++;
	}
	}
	}
	fsum[0] = sume / nume;

	// <H>
	float sumh = 0;
	int numh = 0;
	for (int i = 0; i < Nx + 1; i++) {
	for (int j = 0; j < Ny + 0; j++) {
	for (int k = 0; k < Nz + 0; k++) {
		sumh += (float)fabs(Hx[i + ABC][j + ABC][k + ABC]);
		numh++;
	}
	}
	}
	for (int i = 0; i < Nx + 0; i++) {
	for (int j = 0; j < Ny + 1; j++) {
	for (int k = 0; k < Nz + 0; k++) {
		sumh += (float)fabs(Hy[i + ABC][j + ABC][k + ABC]);
		numh++;
	}
	}
	}
	for (int i = 0; i < Nx + 0; i++) {
	for (int j = 0; j < Ny + 0; j++) {
	for (int k = 0; k < Nz + 1; k++) {
		sumh += (float)fabs(Hz[i + ABC][j + ABC][k + ABC]);
		numh++;
	}
	}
	}
	fsum[1] = sumh / numh;
}


// DFT
static void dft(int ABC, int Nx, int Ny, int Nz, int NRx, int *iRx, int *jRx, int *kRx,
	float ***Ex, float ***Ey, float ***Ez, float fcos, float fsin, float *S_r, float *S_i,
	int iout, float ***Ex_r, float ***Ex_i, float ***Ey_r, float ***Ey_i, float ***Ez_r, float ***Ez_i)
{
	// 受信点(Ez成分)
	for (int n = 0; n < NRx; n++) {
		const int i = iRx[n];
		const int j = jRx[n];
		const int k = kRx[n];
		S_r[n] += fcos * Ez[i + ABC][j + ABC][k + ABC];
		S_i[n] -= fsin * Ez[i + ABC][j + ABC][k + ABC];
	}

	// debug, 全領域調和界, 計算時間が増えるので必要なときのみ
	if (iout) {
		// Ex
		for (int i = 0; i < Nx + 0; i++) {
		for (int j = 0; j < Ny + 1; j++) {
		for (int k = 0; k < Nz + 1; k++) {
			Ex_r[i][j][k] += fcos * Ex[i + ABC][j + ABC][k + ABC];
			Ex_i[i][j][k] -= fsin * Ex[i + ABC][j + ABC][k + ABC];
		}
		}
		}
		// Ey
		for (int i = 0; i < Nx + 1; i++) {
		for (int j = 0; j < Ny + 0; j++) {
		for (int k = 0; k < Nz + 1; k++) {
			Ey_r[i][j][k] += fcos * Ey[i + ABC][j + ABC][k + ABC];
			Ey_i[i][j][k] -= fsin * Ey[i + ABC][j + ABC][k + ABC];
		}
		}
		}
		// Ez
		for (int i = 0; i < Nx + 1; i++) {
		for (int j = 0; j < Ny + 1; j++) {
		for (int k = 0; k < Nz + 0; k++) {
			Ez_r[i][j][k] += fcos * Ez[i + ABC][j + ABC][k + ABC];
			Ez_i[i][j][k] -= fsin * Ez[i + ABC][j + ABC][k + ABC];
		}
		}
		}
	}
}
