/*
txrx.c
送受信点座標
*/

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>

//static void _debug(int, int *, int *, int *, int, int *, int *, int *);

void txrx(int Nout, int Lant, int *Nx, int *Ny, int *Nz,
	int NTx, int *iTx, int *jTx, int *kTx,
	int NRx, int *iRx, int *jRx, int *kRx)
{
/*
	// case-1
	const int mx = 23;
	const int my = 23;
	const int mz = 21;
	//                1   2   3   4   5   6   7   8   9  10  11  12  13  14  15  16
	const int y[] = { 4,  4,  4,  4,  7, 10, 13, 16, 19, 19, 19, 19, 16, 13, 10,  7};
	const int z[] = { 4,  8, 12, 16, 16, 16, 16, 16, 16, 12,  8,  4,  4,  4,  4,  4};
*/

	// case-2
	const int mx = 24;
	const int my = 24;
	const int mz = 21;
	//                1   2   3   4   5   6   7   8   9  10  11  12  13  14  15  16
	const int y[] = { 3,  3,  3,  3,  6,  9, 12, 12, 12, 12, 15, 18, 21, 21, 21, 21};
	const int z[] = { 4,  8, 12, 16, 16, 16, 16, 12,  8,  4,  4,  4,  4,  8, 12, 16};

/*
	// case-3
	const int mx = 22;
	const int my = 22;
	const int mz = 21;
	//                1   2   3   4   5   6   7   8   9  10  11  12  13  14  15  16
	const int y[] = { 6,  6,  6,  6,  6,  6,  6,  6, 16, 16, 16, 16, 16, 16, 16, 16};
	const int z[] = { 3,  5,  7,  9, 11, 13, 15, 17, 17, 15, 13, 11,  9,  7,  5,  3};
*/
/*
	// case-4
	const int mx = 22;
	const int my = 22;
	const int mz = 22;
	//                1   2   3   4   5   6   7   8   9  10  11  12  13  14  15  16
	const int y[] = {11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11};
	const int z[] = { 3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15, 16, 17, 18};
*/

	// 送信点
	for (int itx = 0; itx < NTx; itx++) {
		iTx[itx] = Nout - Lant;
		jTx[itx] = Nout + y[itx];
		kTx[itx] = Nout + z[itx];
	}

	// 受信点
	for (int irx = 0; irx < NRx; irx++) {
		iRx[irx] = Nout + mx + Lant;
		jRx[irx] = jTx[irx];
		kRx[irx] = kTx[irx];
	}

	// FDTDセル数(PML除く)
	*Nx = mx + (2 * Nout);
	*Ny = my + (2 * Nout);
	*Nz = mz + (2 * Nout);

	// check
	assert(NRx == NRx);
	assert((sizeof(y) == NTx * sizeof(int)) && (sizeof(y) == NTx * sizeof(int)));
	for (int itx = 0; itx < NTx; itx++) {
		//printf("Tx: %d %d %d %d\n", itx, iTx[itx], jTx[itx], kTx[itx]);
		assert((iTx[itx] > 0) && (iTx[itx] < *Nx) && (jTx[itx] > 0) && (jTx[itx] < *Ny) && (kTx[itx] > 0) && (kTx[itx] < *Nz));
	}
	for (int irx = 0; irx < NRx; irx++) {
		//printf("Rx: %d %d %d %d\n", irx, iRx[irx], jRx[irx], kRx[irx]);
		assert((iRx[irx] > 0) && (iRx[irx] < *Nx) && (jRx[irx] > 0) && (jRx[irx] < *Ny) && (kRx[irx] > 0) && (kRx[irx] < *Nz));
	}

	// debug
	//_debug(NTx, iTx, jTx, kTx, NRx, iRx, jRx, kRx);
}
/*
// debug
#include <math.h>
#include "../fdtd2d/alloc.h"
#include "../fdtd2d/ev.h"
extern void plot2d_contour(float, float, int, int, float **, float, float, int, int);
static void _debug(
	int NTx, int *iTx, int *jTx, int *kTx,
	int NRx, int *iRx, int *jRx, int *kRx)
{
	float **d;
	alloc2d(float, d, NTx, NRx)
	float dmin = +1e12f;
	float dmax = -1e12f;
	for (int itx = 0; itx < NTx; itx++) {
	for (int irx = 0; irx < NRx; irx++) {
		d[itx][irx] = 1 / (float)sqrt(
			(iTx[itx] - iRx[irx]) * (iTx[itx] - iRx[irx]) +
			(jTx[itx] - jRx[irx]) * (jTx[itx] - jRx[irx]) +
			(kTx[itx] - kRx[irx]) * (kTx[itx] - kRx[irx]));
		if (d[itx][irx] < dmin) {
			dmin = d[itx][irx];
		}
		if (d[itx][irx] > dmax) {
			dmax = d[itx][irx];
		}
		//printf("%d %d %f\n", itx, irx, d[itx][irx]);
	}
	}
	printf("%f %f\n", dmin, dmax);
	ev2d_init(NTx, NRx);
	ev2d_newPage();
	plot2d_contour(0, 0, NTx, NRx, d, 0, dmax, 1, 0);
	ev2d_output();
	exit(0);
}
*/
