/*
writedata.c
*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <assert.h>

// ファイル出力
void writedata(const char outfile[], int ndata, int nx, int ny, int nz, int nout, int ntx, int nrx,
	float ****d, float ***s_r, float ***s_i)
{
	const float inf = 1e12f;

	// (1) 誘電率分布(内部領域)

	const int i0 = nout;
	const int i1 = nx - nout;
	const int j0 = nout;
	const int j1 = ny - nout;
	const int k0 = nout;
	const int k1 = nz - nout;

	// 最小値・最大値
	float dmin = +inf;
	float dmax = -inf;
	for (int idata = 0; idata < ndata; idata++) {
		for (int i = i0; i < i1; i++) {
		for (int j = j0; j < j1; j++) {
		for (int k = k0; k < k1; k++) {
			if (d[idata][i][j][k] < dmin) {
				dmin = d[idata][i][j][k];
			}
			if (d[idata][i][j][k] > dmax) {
				dmax = d[idata][i][j][k];
			}
		}
		}
		}
	}
	//printf("%f %f\n", dmin, dmax);

	// 1D配列化+1バイト化
	const int mdata = ndata * (i1 - i0) * (j1 - j0) * (k1 - k0);
	unsigned char *d1d = (unsigned char *)malloc(mdata * sizeof(unsigned char));
	int m = 0;
	for (int idata = 0; idata < ndata; idata++) {
		for (int i = i0; i < i1; i++) {
		for (int j = j0; j < j1; j++) {
		for (int k = k0; k < k1; k++) {
			d1d[m++] = (unsigned char)(255.0f * (d[idata][i][j][k] - dmin) / (dmax - dmin));
		}
		}
		}
	}
	assert(m == mdata);

	// (2) S行列

	// 1D配列化
	const int kdata = ndata * 2 * ntx * nrx;
	float *s1d = (float *)malloc(kdata * sizeof(float));
	int k = 0;
	for (int idata = 0; idata < ndata; idata++) {
		for (int i = 0; i < ntx; i++) {
		for (int j = 0; j < nrx; j++) {
			s1d[k++] = s_r[idata][i][j];
		}
		}
		for (int i = 0; i < ntx; i++) {
		for (int j = 0; j < nrx; j++) {
			s1d[k++] = s_i[idata][i][j];
		}
		}
	}
	assert(k == kdata);

	// 整数データ
	int idata[6];
	idata[0] = ndata; idata[1] = i1 - i0; idata[2] = j1 - j0; idata[3] = k1 - k0, idata[4] = ntx; idata[5] = nrx;

	// 実数データ
	float fdata[2];
	fdata[0] = dmin; fdata[1] = dmax;

	// ファイル出力
	FILE *fp;
	if ((fp = fopen(outfile, "wb")) == NULL) {
		printf("%s open error\n", outfile);
		return;
	}

	fwrite(idata, sizeof(int),               6, fp);
	fwrite(fdata, sizeof(float),             2, fp);
	fwrite(d1d,   sizeof(unsigned char), mdata, fp);
	fwrite(s1d,   sizeof(float),         kdata, fp);

	fclose(fp);

	free(d1d);
	free(s1d);
}
